
function disco_items ( callback, what, node )
	local request =
		lm.message.create { to = what, mtype = 'iq-get',
			query = { xmlns = 'http://jabber.org/protocol/disco#items' }
		}
	if node then
		request:child( 'query' ):attribute ( 'node', node )
	end
	return lm.connection.bless( main.connection () ):send ( request,
		function ( conn, message )
			if message:child ( 'error' ) then
				callback ( message:child( 'error' ):children():name () )
			else
				local item = message:child( 'query' ):children ()
				local items = { }
				while item do
					if item:name () == 'item' then
						table.insert ( items, { jid = item:attribute ( 'jid' ), node = item:attribute ( 'node' ), name = item:attribute ( 'name' ) } )
					end
					item = item:next ()
				end
				callback ( items )
			end
			return true
		end )
end

function disco_info ( callback, what )
	return lm.connection.bless( main.connection () ):send (
		lm.message.create { to = what, mtype = 'iq-get',
			query = { xmlns='http://jabber.org/protocol/disco#info' }
		},
		function ( conn, message )
			local items_supported = false
			if message:child ( 'error' ) then
				callback ( message:child( 'error' ):children():name () )
			else
				local item = message:child( 'query' ):children ()
				local identities = { }
				local features   = { }
				while item do
					if item:name () == 'identity' then
						table.insert ( identities, { category = item:attribute ( 'category' ), type = item:attribute ( 'type' ), name = item:attribute ( 'name' ) } )
					elseif item:name () == 'feature' then
						table.insert ( features, item:attribute ( 'var' ) )
					end
					item = item:next ()
				end
				callback ( identities, features )
			end
			return true
		end )
end

main.command ( 'disco',
	function ( args )
		local who
		if args.t then
			who = args.t
		else
			who = main.full_jid ()
		end
		if args[1] == 'items' then
			local node = args[2]
			disco_items (
				function ( items )
					if type ( items ) == 'string' then
						main.print_info ( who, ("Items service discovery for %s (%s) failed: %s"):format ( who, node or '', items ) )
					else
						local text = ''
						for index, item in ipairs ( items ) do
							text = text .. ("\n    [%s (%s)] %s"):format ( item.jid or '', item.node or '', item.name or '' )
						end
						if text ~= '' then
							main.print_info ( who, ("Items service discovery result for %s (%s):%s"):format ( who, node or '', text ) )
						else
							main.print_info ( who, ("No items in discovery result for %s (%s)"):format ( who, node or '' ) )
						end
					end
				end, who, node )
		else
			disco_info (
				function ( identities, features )
					if type ( identities ) == 'string' then
						main.print_info ( who, ("Info service discovery for %s failed: %s"):format ( who, identities ) )
					else
						main.print_info ( who, ("Service info discovery result for %s:"):format ( who ) )
						local text = ''
						for index, identity in ipairs ( identities ) do
							text = text .. ("\n    [%s (%s)] %s"):format ( identity.category or '', identity.type or '', identity.name or '' )
						end
						if text ~= '' then
							main.print_info ( who, "  Identities:" .. text )
						else
							main.print_info ( who, "  No identities" )
						end
						text = ''
						for index, feature in ipairs ( features ) do
							text = text .. ("\n    [%s]"):format ( feature or '' )
						end
						if text ~= '' then
							main.print_info ( who, "  Features:" .. text )
						else
							main.print_info ( who, "  No features" )
						end
					end
				end, who )
		end
	end, true, 'jid' )

commands_help['disco'] = "[-t target_jid] [info | items] [node]\n\nService discovery request.\nInfo is sent if omitted.\nIf info reveals, that buddy can do items, items request also will be sent."

-- vim: se ts=4: --
